import {Component, OnInit} from '@angular/core';
import {AngularFireDatabase, SnapshotAction} from '@angular/fire/compat/database';
import {Observable} from 'rxjs';
import {map} from 'rxjs/operators';
import {FactionInBody} from './faction-in-body';
import {Faction} from '../factions/faction';
import {Body} from '../bodies/body';

@Component({
  selector: 'app-factions-in-bodies',
  templateUrl: './factions-in-bodies.component.html',
  styleUrls: ['./factions-in-bodies.component.scss']
})
export class FactionsInBodiesComponent implements OnInit {
  public factionsInBodies: Observable<SnapshotAction<FactionInBody>[]> = new Observable<SnapshotAction<FactionInBody>[]>();
  public bodies: Observable<SnapshotAction<Body>[]> = new Observable<SnapshotAction<Body>[]>();
  public factions: Observable<SnapshotAction<Faction>[]> = new Observable<SnapshotAction<Faction>[]>();

  constructor(private database: AngularFireDatabase) { }

  ngOnInit(): void {
    const factionsInBodiesRef = this.database.list<FactionInBody>('factionsInBodies');
    const factionsRef = this.database.list<Faction>('factions');
    const bodiesRef = this.database.list<Body>('bodies');
    this.factionsInBodies = factionsInBodiesRef.snapshotChanges().pipe(
      map(results => {
        return results.sort(FactionsInBodiesComponent.sortDescending)
      })
    );
    this.factions = factionsRef.snapshotChanges();
    this.bodies = bodiesRef.snapshotChanges();
  }

  public addFactionInBody(body: string, faction: string, size: string): void {
    const factionsInBodiesRef = this.database.list<FactionInBody>('factionsInBodies');
    factionsInBodiesRef.push({
      body: body,
      faction: faction,
      size: +size
    });
  }

  public deleteFactionInBody(key: string|null): void {
    const factionsInBodiesRef = this.database.list<FactionInBody>('factionsInBodies');
    if (key != null) {
      factionsInBodiesRef.remove(key);
    }
  }

  private static sortDescending(factionA: SnapshotAction<FactionInBody>, factionB: SnapshotAction<FactionInBody>): number {
    const factionAValue = factionA.payload.val();
    const factionBValue = factionB.payload.val();

    if (factionAValue == null || factionBValue == null) {
      return 0;
    }

    if (factionAValue.size < factionBValue.size) {
      return 1;
    }
    if (factionAValue.size > factionBValue.size) {
      return -1;
    }

    return 0;
  }

}
